%define anolis_release 5
%global soversion 3

%define srpmhash() %{lua:
local files = rpm.expand("%_specdir/openssl.spec")
for i, p in ipairs(patches) do
   files = files.." "..p
end
for i, p in ipairs(sources) do
   files = files.." "..p
end
local sha256sum = assert(io.popen("cat "..files.." 2>/dev/null | sha256sum"))
local hash = sha256sum:read("*a")
sha256sum:close()
print(string.sub(hash, 0, 16))
}

%global _performance_build 1

Name:           openssl
Version:        3.0.7
Release:        %{anolis_release}%{?dist}
Epoch:          1
Summary:        Utilities from the general purpose cryptography library with TLS implementation

License:        ASL 2.0
URL:            http://www.openssl.org/
Source0:        https://www.openssl.org/source/%{name}-%{version}.tar.gz
Source1:        hobble-openssl
Source2:        Makefile.certificate
Source3:        genpatches
Source4:        make-dummy-cert
Source5:        renew-dummy-cert
Source6:        configuration-switch.h
Source7:        configuration-prefix.h
Source8:        ec_curve.c
Source9:        ectest.c 

Patch1:         0001-Aarch64-and-ppc64le-use-lib64.patch
Patch2:         0002-Use-more-general-default-values-in-openssl.cnf.patch
Patch3:         0003-Do-not-install-html-docs.patch
Patch4:         0004-Override-default-paths-for-the-CA-directory-tree.patch
Patch5:         0005-apps-ca-fix-md-option-help-text.patch
Patch6:         0006-Disable-signature-verification-with-totally-unsafe-h.patch
Patch7:         0007-Add-support-for-PROFILE-SYSTEM-system-default-cipher.patch
Patch8:         0008-Add-FIPS_mode-compatibility-macro.patch
# Add check to see if fips flag is enabled in kernel
Patch9: 	0009-Add-Kernel-FIPS-mode-flag-support.patch
Patch10:        0010-Remove-EC-curves.patch
Patch11:        0011-Disable-explicit-ec.patch
Patch12:        0012-load-legacy-prov.patch
# Tmp: test name change
Patch13: 	0013-tmp-Fix-test-names.patch
# We load FIPS provider and set FIPS properties implicitly
Patch14: 	0014-Force-fips.patch
# Embed HMAC into the fips.so
Patch15: 	0015-FIPS-embed-hmac.patch
# Comment out fipsinstall command-line utility
Patch16: 	0016-fipsinstall_disable.patch
# Skip unavailable algorithms running `openssl speed`
Patch17: 	0017-speed-skip-unavailable-dgst.patch
# Extra public/private key checks required by FIPS-140-3
Patch18: 	0018-FIPS-140-3-keychecks.patch
# Minimize fips services
Patch19: 	0019-FIPS-services-minimize.patch
# Execute KATS before HMAC verification
Patch20: 	0020-FIPS-early-KATS.patch
# Selectively disallow SHA1 signatures
Patch21: 	0021-Selectively-disallow-SHA1-signatures.patch
Patch22: 	0022-FIPS-enable-pkcs12-mac.patch
Patch23: 	0023-Support-different-R_BITS-lengths-for-KBKDF.patch
# Allow SHA1 in seclevel 2 if rh-allow-sha1-signatures = yes
Patch24: 	0024-Allow-SHA1-in-seclevel-2-if-rh-allow-sha1-signatures.patch
# https://github.com/openssl/openssl/pull/18103
# The patch is incorporated in 3.0.3 but we provide this function since 3.0.1
# so the patch should persist
Patch25: 	0025-strcasecmp.patch
Patch26: 	0026-FIPS-limit-rsa-encrypt.patch
Patch27: 	0027-FIPS-KAT-signature-tests.patch
Patch28: 	0028-Deny-SHA-1-signature-verification-in-FIPS-provider.patch
Patch29: 	0029-fips-Expose-a-FIPS-indicator.patch
Patch30: 	0030-AES-GCM-performance-optimization.patch
# https://bugzilla.redhat.com/show_bug.cgi?id=2102535
Patch31: 	0031-FIPS-Use-OAEP-in-KATs-support-fixed-OAEP-seed.patch
# https://bugzilla.redhat.com/show_bug.cgi?id=2102535
Patch32: 	0032-FIPS-Use-digest_sign-digest_verify-in-self-test.patch
Patch33: 	0033-FIPS-Use-FFDHE2048-in-self-test.patch
# Downstream only. Reseed DRBG using getrandom(GRND_RANDOM)
Patch34: 	0034-FIPS-140-3-DRBG.patch
Patch35: 	0035-FIPS-140-3-zeroization.patch
Patch36: 	0036-Add-FIPS-indicator-parameter-to-HKDF.patch
Patch37: 	0037-rand-Forbid-truncated-hashes-SHA-3-in-FIPS-prov.patch
Patch38: 	0038-signature-Remove-X9.31-padding-from-FIPS-prov.patch
Patch39: 	0039-kbkdf-Add-explicit-FIPS-indicator-for-key-length.patch
Patch40: 	0040-hmac-Add-explicit-FIPS-indicator-for-key-length.patch
Patch41: 	0041-pbkdf2-Set-minimum-password-length-of-8-bytes.patch
Patch42: 	0042-FIPS-RSA-disable-shake.patch
Patch43: 	0043-signature-Add-indicator-for-PSS-salt-length.patch
Patch44: 	0044-PSS-salt-length-from-provider.patch
Patch45: 	0045-signature-Clamp-PSS-salt-len-to-MD-len.patch
Patch46: 	0046-FIPS-RSA-encapsulate.patch
Patch47: 	0047-provider-improvements.patch

BuildRequires:  gcc git coreutils perl-interpreter sed zlib-devel /usr/bin/cmp
BuildRequires:  lksctp-tools-devel
BuildRequires:  /usr/bin/rename
BuildRequires:  /usr/bin/pod2man
BuildRequires:  /usr/sbin/sysctl
BuildRequires:  perl(Test::Harness) perl(Test::More) perl(Math::BigInt)
BuildRequires:  perl(Module::Load::Conditional) perl(File::Temp)
BuildRequires:  perl(Time::HiRes) perl(IPC::Cmd) perl(Pod::Html) perl(Digest::SHA)
BuildRequires:  perl(FindBin) perl(lib) perl(File::Compare) perl(File::Copy) perl(bigint)
BuildRequires:  perl(ExtUtils::MakeMaker)
Requires:       coreutils
Requires:       %{name}-libs = %{epoch}:%{version}-%{release}

%description
The OpenSSL toolkit provides support for secure communications between
machines. OpenSSL includes a certificate management tool and shared
libraries which provide various cryptographic algorithms and
protocols.

%package libs
Summary:        A general purpose cryptography library with TLS implementation
Requires:       ca-certificates >= 2008-5
Requires:       crypto-policies >= 20180730
Recommends:     openssl-pkcs11 = %{epoch}:%{version}-%{release}

%description libs
OpenSSL is a toolkit for supporting cryptography. The openssl-libs
package contains the libraries that are used by various applications which
support cryptographic algorithms and protocols.

%package devel
Summary:        Files for development of applications which will use OpenSSL
Requires:       %{name}-libs = %{epoch}:%{version}-%{release}
Requires:       pkgconfig

%description devel
OpenSSL is a toolkit for supporting cryptography. The openssl-devel
package contains include files needed to develop applications which
support various cryptographic algorithms and protocols.

%package perl
Summary:        Perl scripts provided with OpenSSL
Requires:       perl-interpreter
Requires:       %{name} = %{epoch}:%{version}-%{release}

%description perl
OpenSSL is a toolkit for supporting cryptography. The openssl-perl
package provides Perl scripts for converting certificates and keys
from other formats to the formats used by the OpenSSL toolkit.

%package doc
Summary:        Doc files for OpenSSL
Requires:       %{name} = %{epoch}:%{version}-%{release}
BuildArch:      noarch

%description doc
Doc files for OpenSSL

%prep
%autosetup -S git
%{SOURCE1} > /dev/null
cp %{SOURCE8} crypto/ec/
cp %{SOURCE9} test/

%build
sslarch=%{_os}-%{_target_cpu}
%ifarch x86_64
sslflags=enable-ec_nistp_64_gcc_128
%endif
%ifarch aarch64
sslarch=linux-aarch64
sslflags=enable-ec_nistp_64_gcc_128
%endif
%ifarch riscv64
sslarch=linux-generic64
%endif

RPM_OPT_FLAGS="$RPM_OPT_FLAGS -Wa,--noexecstack -Wa,--generate-missing-build-notes=yes -DPURIFY $RPM_LD_FLAGS"

export HASHBANGPERL=/usr/bin/perl
%define fips %{version}-%{srpmhash}

./Configure \
        --prefix=%{_prefix} --openssldir=%{_sysconfdir}/pki/tls ${sslflags} \
        --system-ciphers-file=%{_sysconfdir}/crypto-policies/back-ends/openssl.config \
        zlib enable-camellia enable-seed enable-rfc3779 enable-sctp \
        enable-cms enable-md2 enable-rc5 enable-ktls enable-fips\
        no-mdc2 no-ec2m enable-buildtest-c++\
        shared  ${sslarch} $RPM_OPT_FLAGS '-DDEVRANDOM="\"/dev/urandom\"" -DREDHAT_FIPS_VERSION="\"%{fips}\""'\
	-Wl,--allow-multiple-definition

make -s %{?_smp_mflags} all

for i in libcrypto.pc libssl.pc openssl.pc ; do
  sed -i '/^Libs.private:/{s/-L[^ ]* //;s/-Wl[^ ]* //}' $i
done

%install
rm -rf $RPM_BUILD_ROOT
install -d $RPM_BUILD_ROOT{%{_bindir},%{_includedir},%{_libdir},%{_mandir},%{_libdir}/openssl,%{_pkgdocdir}}
%make_install
rename so.%{soversion} so.%{version} $RPM_BUILD_ROOT%{_libdir}/*.so.%{soversion}
for lib in $RPM_BUILD_ROOT%{_libdir}/*.so.%{version} ; do
        chmod 755 ${lib}
        ln -s -f `basename ${lib}` $RPM_BUILD_ROOT%{_libdir}/`basename ${lib} .%{version}`
        ln -s -f `basename ${lib}` $RPM_BUILD_ROOT%{_libdir}/`basename ${lib} .%{version}`.%{soversion}
done

rm -f $RPM_BUILD_ROOT%{_libdir}/*.a

mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/pki/tls/certs
install -m644 %{SOURCE2} $RPM_BUILD_ROOT%{_pkgdocdir}/Makefile.certificate
install -m755 %{SOURCE4} $RPM_BUILD_ROOT%{_bindir}/make-dummy-cert
install -m755 %{SOURCE5} $RPM_BUILD_ROOT%{_bindir}/renew-dummy-cert

mv $RPM_BUILD_ROOT%{_sysconfdir}/pki/tls/misc/*.pl $RPM_BUILD_ROOT%{_bindir}
mv $RPM_BUILD_ROOT%{_sysconfdir}/pki/tls/misc/tsget $RPM_BUILD_ROOT%{_bindir}
mv $RPM_BUILD_ROOT%{_mandir}/man5/config.5ossl $RPM_BUILD_ROOT%{_mandir}/man5/config.cnf.5

mkdir -p -m755 $RPM_BUILD_ROOT%{_sysconfdir}/pki/CA/{certs,crl,newcerts}
mkdir -m700 $RPM_BUILD_ROOT%{_sysconfdir}/pki/CA/private

touch -r %{SOURCE2} $RPM_BUILD_ROOT%{_sysconfdir}/pki/tls/openssl.cnf
touch -r %{SOURCE2} $RPM_BUILD_ROOT%{_sysconfdir}/pki/tls/ct_log_list.cnf

rm -f $RPM_BUILD_ROOT%{_sysconfdir}/pki/tls/{openssl.cnf.dist,ct_log_list.cnf.dist,fipsmodule.cnf}

%files
%{!?_licensedir:%global license %%doc}
%license LICENSE.txt
%{_bindir}/make-dummy-cert
%{_bindir}/renew-dummy-cert
%{_bindir}/openssl
%{_mandir}/man1/*
%{_mandir}/man5/*
%{_mandir}/man7/*
%{_pkgdocdir}/Makefile.certificate
%exclude %{_mandir}/man1/*.pl*
%exclude %{_mandir}/man1/tsget*

%files libs
%{!?_licensedir:%global license %%doc}
%license LICENSE.txt
%dir %{_sysconfdir}/pki/tls
%dir %{_sysconfdir}/pki/tls/certs
%dir %{_sysconfdir}/pki/tls/misc
%dir %{_sysconfdir}/pki/tls/private
%config(noreplace) %{_sysconfdir}/pki/tls/openssl.cnf
%config(noreplace) %{_sysconfdir}/pki/tls/ct_log_list.cnf
%attr(0755,root,root) %{_libdir}/libcrypto.so.%{version}
%{_libdir}/libcrypto.so.%{soversion}
%attr(0755,root,root) %{_libdir}/libssl.so.%{version}
%{_libdir}/libssl.so.%{soversion}
%attr(0755,root,root) %{_libdir}/engines-%{soversion}
%attr(0755,root,root) %{_libdir}/ossl-modules

%files devel
%doc CHANGES.md doc/dir-locals.example.el doc/openssl-c-indent.el
%{_prefix}/include/openssl
%{_libdir}/*.so
%{_mandir}/man3/*
%{_libdir}/pkgconfig/*.pc

%files perl
%{_bindir}/c_rehash
%{_bindir}/*.pl
%{_bindir}/tsget
%{_mandir}/man1/*.pl*
%{_mandir}/man1/tsget*
%dir %{_sysconfdir}/pki/CA
%dir %{_sysconfdir}/pki/CA/private
%dir %{_sysconfdir}/pki/CA/certs
%dir %{_sysconfdir}/pki/CA/crl
%dir %{_sysconfdir}/pki/CA/newcerts

%files doc
%doc NEWS.md README.md

%ldconfig_scriptlets libs

%changelog
* Thu Feb 16 2023 happy_orange <songnannan@linux.alibaba.com> - 1:3.0.7-5
- Enalbe sm2 and sm4

* Thu Feb 02 2023 happy_orange <songnannan@linux.alibaba.com> - 1:3.0.7-4
- add some patches

* Fri Dec 23 2022 chen_yu_ao <m15927509890@163.com> - 1:3.0.7-3
- change patch numbers and format

* Thu Nov 03 2022 Funda Wang <fundawang@yeah.net> - 1:3.0.7-2
- Enalbe sm2 and sm4

* Thu Nov 03 2022 Chunmei Xu <xuchunmei@linux.alibaba.com> - 1:3.0.7-1
- update to 3.0.7

* Mon Aug 29 2022 Chunmei Xu <xuchunmei@linux.alibaba.com> - 1:3.0.5-1
- update to 3.0.5

* Mon Mar  21 2022 forrest_ly <flin@linux.alibaba.com> - 1:3.0.2-1
- Update to 3.0.2

* Wed Mar  16 2022 forrest_ly <flin@linux.alibaba.com> - 1:3.0.1-2
- Fix Epoch for others pkg deps

* Tue Mar  8 2022 forrest_ly <flin@linux.alibaba.com> - 3.0.1-1
- Init for Anolis OS 23 
